# -*- coding: utf-8 -*-
"""
Created on Sun May 21 08:51:33 2023

@author: atakan
"""


import carbatpy as cb


def throttle(p_out, fluid, calc_type="const_h", name="throttle"):
    """
    throttle output state calculation

    so far only for a constant enthalpy

    Parameters
    ----------
    p_out : float
        output pressure.
    fluid : fprop.Fluid
        entering fluid, including properties, composition, and model.
    calc_type : string, optional
        how to calculate, so far, only one implemented. The default is
        "const_h".
    name : string, optional
        name of the device. The default is "throttle".

    Returns
    -------
    state_out : array of float
        compressor output state containing [T,p,h,v,s,q].

    """
    if calc_type == "const_h":
        state_out = fluid.set_state([fluid.properties.enthalpy, p_out], "HP")
    else:
        raise Exception(
            f"The option{calc_type} is not yet implemented for throttles")
    return state_out


if __name__ == "__main__":
    FLUID = "Propane * Pentane"
    comp = [.50, 0.5]
    flm = cb.fprop.FluidModel(FLUID)
    myFluid = cb.fprop.Fluid(flm, comp)
    state_in = myFluid.set_state([320., 19e5], "TP")

    P_OUT = 5e5

    state_out_main = throttle(P_OUT, myFluid)
    print("Throttle:\nInput:", state_in, "\nOutput:",
          state_out_main, "\nDifference", state_out_main-state_in)



"""Main module."""
__version__ = "0.1.1"


# from config import _T_SURROUNDING, _P_SURROUNDING, _RESULTS_DIR 
# import src.models as models
# import src.utils as utils
# import src.helpers as helpers

# from src.models.fluids import fluid_props as fprop
# from src.models.coupled import heat_pump_simple_v2 as hp_simple
# from src.models.components import compressor_simple, throttle_simple
# from src.models.components import heat_exchanger_thermo_v2 as hex_th
# from src.utils import exergy_loss as exlo
# from src.helpers import file_copy


# import src.models.fluids.fluid_props as fprop
# from src.models.coupled import heat_pump_simple_v2 as hp_simple
# from src.models.components import compressor_simple, throttle_simple
# from src.models.components import heat_exchanger_thermo_v2 as hex_th
# from src.utils import exergy_loss as exlo

#_T_SURROUNDING = 288.15 # K






# -*- coding: utf-8 -*-
"""
first test for fluid


Created on Sun Oct 15 18:07:07 2023

@author: atakan
"""


import unittest
import carbatpy as cb
# import src.models.fluids.fluid_props as fprop
# import src.models.components.throttle_simple as throt


class TestFluidModule(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        """We only want to pull this data once for each TestCase since it is
        an expensive operation"""

        fluid_mixture = "Propane * Pentane"
        self.flm = cb.fprop.FluidModel(fluid_mixture)

    def test_fluid_values(self):
        """ensures that the expected enthalpies are correct"""
        fluid_mixture = "Propane * Pentane"
        self.flm = cb.fprop.FluidModel(fluid_mixture)
        print(self.flm.fluid)
        comp = [.50, 0.5]
        self.my_fluid = cb.fprop.Fluid(self.flm, comp)
        st0 = self.my_fluid.set_state([300., 1e5], "TP")
        self.assertAlmostEqual(self.my_fluid.properties.enthalpy,
                               453021.3769789692, places=14)
        # transport properties, speed od sound
        st1 = self.my_fluid.set_state([300., 1e5], "TP",
                                cb.fprop._TRANS_STRING)
        self.assertAlmostEqual(self.my_fluid.properties.speed_of_sound,
                               2.1158293060385915e+02, places=14)

        # fluid, different composition
        st0 = self.my_fluid.set_state([300., 1e5], "TP", composition=[.35, .65])
        self.assertAlmostEqual(self.my_fluid.properties.enthalpy,
                               414499.55081964, places=14)
        
        mean_temp_act = self.my_fluid.calc_temp_mean(st0[2]+1e5)
        self.assertAlmostEqual(mean_temp_act,
                               327.2548125203546, places=14)

        # Water
        self.flm = cb.fprop.FluidModel("Water")
        self.my_fluid = cb.fprop.Fluid(self.flm, [1])
        st0 = self.my_fluid.set_state([300., 1e5], "TP")
        self.assertAlmostEqual(self.my_fluid.properties.enthalpy,
                               112653.67968890067, places=14)
        print(self.flm.fluid)

    def test_throttle(self):
        # throttle:
        fluid_mixture = "Propane * Pentane"
        self.flm = cb.fprop.FluidModel(fluid_mixture)
        print(self.flm.fluid)
        comp = [.50, 0.5]
        self.my_fluid = cb.fprop.Fluid(self.flm, comp)
        state_in = self.my_fluid.set_state([320., 19e5], "TP")
        p_out = 5e5
        state_out = cb.throttle_simple.throttle(p_out, self.my_fluid)
        self.assertAlmostEqual(self.my_fluid.properties.enthalpy,
                               143230.6314641813, places=14)

    def test_heat_pump_simple_v2(self):


        fluid_mixture = "Propane * Ethane * Pentane *Isobutane"
        # comp = [.75, 0.05, 0.15, 0.05]
        x1 = 0.25
        x2 = 0.05
        x3 = 0.05
        x4 = 1 - x1-x2-x3
        comp = [x1, x2, x3, x4]  # [0.164,.3330,.50300,0.0]
        print(f"{fluid_mixture}, composition:\n{comp}")
        temp_surrounding = 273.15+20
        fluid_storage = "Water"  #
        fluid_cold_storage = "Water"  # "Methanol"  # "Water"  #

        flm = cb.fprop.FluidModel(fluid_mixture)
        my_fluid = cb.fprop.Fluid(flm, comp)

        sec_fluid_model = cb.fprop.FluidModel(fluid_storage)
        sec_fluid = cb.fprop.Fluid(sec_fluid_model, [1.])

        cold_fluid_model = cb.fprop.FluidModel(fluid_cold_storage)
        cold_fluid = cb.fprop.Fluid(cold_fluid_model, [1.])

        # Condenser(c) and storage (s), secondary fluids fix all, temperatures(T in K),
        # pressures (p in Pa)
        _eta_s_ = 0.57  # interesting when changed from 0.69 to 0.65, the efficiency
        # decreases, the reason is the low quality along throtteling then
        _storage_temp_in = 273.15 + 50
        _cold_storage_temp_in = temp_surrounding
        _storage_temp_out_ = 273.15 + 60  # 395.0
        _cold_storage_temp_out_ = 284.15
        _storage_pres_in_ = 5e5
        _cold_storage_pres_in_ = 5e5
        _q_dot_required_ = 1e3  # and heat_flow rate (W)
        _D_T_SUPER_ = 15  # super heating of working fluid
        _temp_diff_min_ = 4.  # minimum approach temperature (pinch point)
        # high T-storages
        state_sec_out = sec_fluid.set_state(
            [_storage_temp_out_, _storage_pres_in_], "TP")
        state_sec_in = sec_fluid.set_state(
            [_storage_temp_in, _storage_pres_in_], "TP")

        #  low T sorages:
        state_cold_out = cold_fluid.set_state(
            [_cold_storage_temp_out_, _cold_storage_pres_in_], "TP")
        state_cold_in = cold_fluid.set_state(
            [_cold_storage_temp_in, _cold_storage_pres_in_], "TP")

        # working fluid
        T_DEW = _storage_temp_out_  # + _temp_diff_min_
        state_in_cond = my_fluid.set_state(
            [T_DEW, 1.], "TQ")  # find high pressure
        state_out_cond = my_fluid.set_state([_storage_temp_in + _temp_diff_min_,
                                            state_in_cond[1]], "TP")
        state_satv_evap = my_fluid.set_state(
            [_storage_temp_in-_temp_diff_min_-_D_T_SUPER_, 1.], "TQ")  # find minimum pressure
        p_low = state_satv_evap[1]

        T_IN = _storage_temp_in - _temp_diff_min_

        state_out_evap = my_fluid.set_state([p_low,
                                            T_IN], "PT")

        FIXED_POINTS = {"eta_s": _eta_s_,
                        "p_low": state_out_evap[1],
                        "p_high": state_in_cond[1],
                        "T_hh": _storage_temp_out_,
                        "h_h_out_sec": state_sec_out[2],
                        "h_h_out_w": state_out_cond[2],
                        "h_l_out_cold": state_cold_out[2],
                        "h_l_out_w": state_out_evap[2],
                        "T_hl": _storage_temp_in,
                        "T_lh": _storage_temp_in,
                        "T_ll": _cold_storage_temp_out_,  # 256.0,
                        "Q_dot_h": _q_dot_required_,
                        "d_temp_min": _temp_diff_min_}

        print(
            f"p-ratio: {state_in_cond[1]/state_out_evap[1]: .2f}, p_low: {state_out_evap[1]/1e5: .2} bar")
        hp0 = cb.hp_simple.HeatPump(
            [my_fluid, sec_fluid, cold_fluid], FIXED_POINTS)
        cop = hp0.calc_p_high(FIXED_POINTS["p_high"], verbose=False)
        hp0.hp_plot()
        out = hp0.evaluation
        print(
            f"COP: {cop},p-ratio: {out['p_high']/out['p_low']:.2f}, p_low {out['p_low']/1e5:.2f} bar")
        print(
            f"COP: {cop},p-ratio: {out['p_high']/out['p_low']}, p_low {out['p_low']/1e5}")
        print(
            f'exergy loss rate: {out["exergy_loss_rate"]}, eff: {1-out["exergy_loss_rate"]/out["Power"]:.4f}')

        self.assertAlmostEqual(cop,
                               3.2679531973379765, places=14)
        self.assertAlmostEqual(out["exergy_loss_rate"],
                               185.3145101271498, places=14)
        self.assertAlmostEqual(1-out["exergy_loss_rate"]/out["Power"],
                               0.39440085411686,
                               places=14)

    def test_fluid_screening(self):
        # mixture search:
        # ["DME", "Ethane", "Butane","CO2"]
        fluids_all = ["Propane", "Ethane", "Pentane", "Butane"]
        T_low = 285.00
        T_high = 363.00
        p_low = 10e4
        p_high = 22e5
        dir_name = cb._RESULTS_DIR + r"\testing"
        temp_limit = True
        results = cb.property_eval_mixture.mixture_search(fluids_all, [T_low, T_high], [p_low, p_high],
                                                          dir_name, resolution=21, temp_limit=temp_limit)
        self.assertAlmostEqual(results["results_DataFrame"]["COP_is"].max(),
                               3.7777223672974976, places=14)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
functions for compressor and expander output state calcultations

so far/here: only for fixed isentropic efficiencies

((Part of carbatpy.))
Created on Sun May 21 08:51:33 2023

@author: atakan
"""

import carbatpy as cb


def compressor(p_out, eta_s, fluid, calc_type="const_eta",
               name="compressor"):
    """
    compressor or expander output state calculation

    so far only for a constant isentropic efficiency, according to the pressure
    change an expansion or compression is detected and handled.

    Parameters
    ----------

    p_out : float
        output pressure.
    eta_s : float
        isentropic efficiency.
    fluid : fprop.Fluid
        entering fluid, including properties, composition, and model.
    calc_type : string, optional
        how to calculate, so far, only one implemented. The default is
        "const_eta".
    name : string, optional
        name of the device. The default is "compressor".

    Returns
    -------
    state_out : array of float
        compressor output state containing [T,p,h,v,s,q].
    work_specific : float
        work per kg of fluid, positive for compressor; units:J/kg.

    """
    expander = False
    if fluid.properties.pressure > p_out:
        expander = True

    if calc_type == "const_eta":
        h_in = fluid.properties.enthalpy

        fluid.set_state(
            [fluid.properties.entropy, p_out], "SP")

        diff_enthalpy_s = fluid.properties.enthalpy-h_in

        if expander:
            work_specific = diff_enthalpy_s * eta_s
        else:
            work_specific = diff_enthalpy_s / eta_s

        state_out = fluid.set_state([h_in + work_specific, p_out], "HP")
    else:
        raise Exception(
            f"The option{calc_type} is not yet implemented for compressors")
    return state_out, work_specific



def pump(p_out, eta_s, fluid, calc_type="const_eta", name="pump"):
    """
    Calculate the exit state of a pump assuming an incompressible fluid.

    Only formulated for constant isentropic efficiency

    Parameters
    ----------
    p_out : float
        output pressure.
    eta_s : float
        isentropic efficiency.
    fluid : fprop.Fluid
        entering fluid, including properties, composition, and model.
    calc_type : string, optional
        how to calculate, so far, only one implemented. The default is
        "const_eta".
    name : string, optional
        name of the device. The default is "pump".

    Returns
    -------
    state_out : array of float
        compressor output state containing [T,p,h,v,s,q].
    work_specific : float
        work per kg of fluid, positive for compressor; units:J/kg.

    """
    if calc_type == "const_eta":
        state_in_ = fluid.properties.state
        work_is = state_in_[3] * (p_out - state_in_[1])
        if work > 0:
            work_specific =  work_is / eta_s
        else:
            work_specific =  work_is * eta_s
        h_out = state_in_[2] + work_specific
        state_out = fluid.set_state([h_out, p_out], "HP")
    else:
        raise Exception(
            f"The option{calc_type} is not yet implemented for compressors")

    return state_out, work_specific


if __name__ == "__main__":

    FLUID = "Propane * Pentane"
    comp = [.80, 0.2]
    flm = cb.fprop.FluidModel(FLUID)
    myFluid = cb.fprop.Fluid(flm, comp)
    P_LOW = 1e5
    T_IN = 310.
    DT_IN_LIQ = -5
    state_in = myFluid.set_state([T_IN, P_LOW], "TP")
    P_OUT = 10e5
    ETA_S = .7

    # Compressor-------------
    state_o, work = compressor(P_OUT, ETA_S, myFluid)
    print(myFluid.properties.temperature, work)
    print("\nCompressor", state_in, "\n", state_o, "\n", state_o-state_in)
    state_in = state_o
    state_o,work = compressor(P_LOW, ETA_S, myFluid)
    print("\nExpander:", state_in, "\n", state_o, "\n", state_o-state_in, work)

    # Pump, incompressible:

    state_in = myFluid.set_state([P_OUT, 0], "PQ")
    state_in = myFluid.set_state([P_OUT, state_in[0]+DT_IN_LIQ], "PT")
    state_o,work = pump(P_LOW, ETA_S, myFluid)
    print(f"pump work: {work:.3f} J/kg, state:{state_o}")



"""Top-level package for carbatpy."""

__author__ = """Burak Atakan"""
__email__ = 'burak.atakan@uni-due.de'
__version__ = '0.1.3'

# __all__ =[]
import os
import sys
import pandas as pd

pd.set_option("mode.copy_on_write", True)

sys.path.insert(0,os.path.abspath('../carbatpy'))
sys.path.insert(0,os.path.abspath('..'))

try:
    from cb_config import _T_SURROUNDING, _RESULTS_DIR, \
                            _P_SURROUNDING,_CARBATPY_BASE_DIR
except:
    from .cb_config import _T_SURROUNDING, _RESULTS_DIR, \
                            _P_SURROUNDING,_CARBATPY_BASE_DIR
    


import src.models as models
import src.utils as utils
import src.helpers as helpers


from src.utils import exergy_loss as exlo
from src.helpers import file_copy

from src.models.fluids import fluid_props as fprop

from src.models.components import compressor_simple, throttle_simple
from src.models.components import heat_exchanger_thermo_v2 as hex_th

from src.models.coupled import heat_pump_simple_v2 as hp_simple
from src.models.coupled import orc_simple_v2 as orc_simple
from src.models.coupled import read_cycle_structure
from src.utils import curve_min_distance_finder, property_eval_mixture, optimize




# -*- coding: utf-8 -*-
"""
Created on Mon May 22 21:09:12 2023

@author: atakan
"""

import os
# from time import time
from ctREFPROP.ctREFPROP import REFPROPFunctionLibrary
import numpy as np
# import CoolProp.CoolProp as CP


# os.environ['RPPREFIX'] = r'C:/Program Files (x86)/REFPROP'
# os.environ['RPPREFIXs'] = r'C:/Program Files (x86)/REFPROP/secondCopyREFPROP'
_PROPS = "REFPROP"  # or "CoolProp"

_fl_properties_names = ("Temperature", "Pressure", "spec_Enthalpy",
                        "spec_Volume", "spec_Entropy", "quality",
                        "spec_internal_Energy",
                        "viscosity", "thermal_conductivity",
                        "Prandtl_number", "k_viscosity", "molecular_mass",
                        "speed_of_sound")
_THERMO_STRING = "T;P;H;V;S;QMASS;E"
_TRANS_STRING = _THERMO_STRING + ";VIS;TCX;PRANDTL;KV;M;W"
_TV_STRING = "T;V"
_T_SURROUNDING = 288.15 # K

# order for coolprop,alle_0:[_temp, p,  h, 1/ rho, s,x,cp, mu,  lambda_s,
# prandtl, phase]"
_UNITS = 21
rp_instance = ""
if _PROPS == "REFPROP":
    try:
        rp_instance = REFPROPFunctionLibrary(os.environ['RPPREFIX'])
        # be careful pressure is in Pa!
        _UNITS = rp_instance.GETENUMdll(0, "MASS BASE SI").iEnum
    except:
        print("print refprop not installed!")



class FluidModel:

    def __init__(self, fluid, units=_UNITS, props=_PROPS, rp_inst=rp_instance):
        """
        For Class Fluidmodell a fluid (mixture) must be defined, the evaluation
        takes place with props, units can be set and an instance can be set,
        the latter is important, if more than one fluid is used.

        Parameters
        ----------
        fluid : String
            as defined in the props (Model), for RefProp it is
            "fluid1 * fluid2 * fluid3".
        units : integer, optional
            units, normally SI wuth MASS base, see props(RefProp).
            The default is _UNITS.
        props : string, optional
            select the property model. The default is _PROPS.
        rp_inst : RefProp-Instance, optional
            where Refprop is installed, for two fluids, as in heat exchangers,
            two installations/instances are needed. The default is rp_instance.

        Returns
        -------
        None.

        """
        self.fluid = fluid
        self.props = props
        self.units = units
        self.rp_instance = rp_inst
        self.set_rp_fluid()

    def set_rp_fluid(self, modwf=REFPROPFunctionLibrary, name='RPPREFIX'):
        """
        A new instance of Refpropdll for the given fluid. It can then be called
        using fluid =""

        Parameters
        ----------
        fluid : string
            fluid (mixture) name, as described in REFPROP.

        Returns
        -------
        self.rp_instance : REFPROP Instance
            for further usage.

        """

        self.rp_instance = modwf(os.environ[name])
        self.rp_instance.SETPATHdll(os.environ[name])
        ierr = self.rp_instance.SETFLUIDSdll(self.fluid)
        if ierr != 0:
            print(f"Fehler in setfluid {ierr}")
            print(self.rp_instance.ERRMSGdll(ierr))
        return self.rp_instance


class FluidState:
    def __init__(self, state):
        self.temperature = state.Output[0]
        self.pressure = state.Output[1]
        self.sp_volume = state.Output[3]
        self.enthalpy = state.Output[2]
        self.entropy = state.Output[4]
        self.quality = state.Output[5]
        self.int_energy = state.Output[6]
        self.state = state.Output[:7]
        self.prop_names = _fl_properties_names


class FluidStateTV:
    def __init__(self, state):
        self.temperature = state.Output[0]
        self.sp_volume = state.Output[1]
        self.state = state.Output[:1]


class FluidStateTransport(FluidState):
    def __init__(self, state):
        super().__init__(state)
        # ";VIS;TCX;PRANDTL;KV;M;W"
        self.viscosity = state.Output[7]
        self.thermal_conductivity = state.Output[8]
        self.prandtl = state.Output[9]
        self.kin_viscosity = state.Output[10]
        self.molecular_mass = state.Output[11]
        self.speed_of_sound =state.Output[12]
        self.transport = state.Output[7:13]
        self.state = state.Output[:13]


class Fluid:
    """ 
    The Fluid class is used to set, get, and print states of  a fluid with a given
    
    model (e.g. RefProp). The compounds are set in the fluidmodel, while the
    composition is also set here.
    
    """

    def __init__(self, fluidmodel, composition=[1.0],
                 option=1):
        self.fluidmodel = fluidmodel
        self.composition = composition
        self.option = option
        self.no_compounds = len(composition)
        self.herr = 0

    def set_composition(self, composition):
        self.composition = composition

    def set_state(self, values, given="TP",
                  wanted=_THERMO_STRING,
                  composition=[]):
        if composition == []:
            composition = self.composition
        else:
            self.composition = composition

        if self.fluidmodel.props == "REFPROP":
            state = self.fluidmodel.rp_instance.REFPROP2dll(
                self.fluidmodel.fluid, given, wanted,
                self.fluidmodel.units,
                0, values[0], values[1],
                composition)

            if state.ierr == 0:
                if wanted == _THERMO_STRING:
                    # print(state)
                    self.properties = FluidState(state)
                elif wanted == _TRANS_STRING:
                    self.properties = FluidStateTransport(state)
                elif wanted == _TV_STRING:
                    self.properties = FluidStateTV(state)
                else:

                    raise Exception(f"properties{wanted} not implemented yet!")
            else:
                self.herr = state.herr
                raise Exception(f"Property-Refprop problem: {state.herr}!")
        else:
            raise Exception(
                f"Property model {self.fluidmodel.props} not implemented yet!")
        return np.array([*self.properties.state])

    def set_state_v(self, values, given="TP", wanted=_THERMO_STRING):
        dimension = np.shape(values)
        number_wanted = wanted.count(";")+1
        output = np.zeros((dimension[0], number_wanted))
        for count, value in enumerate(values):
            output[count, :] = self.set_state(value, given, wanted)
        self.state_v = output
        return output

    def print_state(self):
        pr = self.properties
        flm = self.fluidmodel
        print(f"\n{flm.fluid}, composition: {self.composition}")
        print(f"T:{pr.temperature:.2f} K, p: {pr.pressure/1e5 :.2f} bar,  h: {pr.enthalpy/1000: .2f} kJ/kg, s: {pr.entropy/1000:.3f} kJ/kg K\n")
        if pr.quality >= 0:
            print(f"Quality: {pr.quality :.3f}")
            
    def calc_temp_mean(self, h_final):
        """
        Calculate the thermodynamic mean temperature between the actual state
        
        and the final enthalpy along an isobaric (Delta h /Delta s)

        Parameters
        ----------
        h_final : float
            enthalpy of he final stat.

        Returns
        -------
        temp_mean : float
            the thermodynamic mean temperature.

        """
        actual_props = self.properties.state
        final_props = self.set_state([h_final, actual_props[1]], "HP")
        temp_mean = (final_props[2] - actual_props[2]) \
            / (final_props[4] - actual_props[4]) 
        return temp_mean
    

if __name__ == "__main__":
    FLUID = "Propane * Pentane"
    comp = [.50, 0.5]
    flm = FluidModel(FLUID)
    myFluid = Fluid(flm, comp)
    st0 = myFluid.set_state([300., 1e5], "TP")
    st1 = myFluid.set_state([300., 1e5], "TP",
                            _TRANS_STRING)
    print(st0, st1)
    myFluid.print_state()
    myFluid.set_composition([.2, .8])
    st0 = myFluid.set_state([300., 1e5], "TP", composition=[.35, .65])
    myFluid.print_state()
    
    mean_temp_act = myFluid.calc_temp_mean(st0[2]+1e5)
    print(f"Mean Temperature {mean_temp_act} K")

    # value_vec = np.array([[300, 1e5], [400, 1e5], [500, 1e5]])
    # stv = myFluid.set_state_v(value_vec, "TP")

    # print(myFluid.set_state_v(value_vec, "TP"))
    # print(myFluid.set_state([300., 1.], "TQ"))



#!/usr/bin/env python

"""Tests for `carbatpy` package."""

import os
import unittest
import numpy as np
import pandas as pd

import carbatpy as cbp


class TestCarbatpy(unittest.TestCase):
    """Tests for `carbatpy` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_pareto_opt_screened(self):
        """Test the pareto function within utils.optimize."""

        directory = cbp._CARBATPY_BASE_DIR
        filename = directory + r"\\tests\\test_files\\"
        filename += r"test_data_ProEthPenBut\2024-02-05-08-37-ProEthPenBut.csv"
        objectives = objectives_act = ['p_ratio', 'T_glide_h', 'COP_is']
        sense = ["min", "min", "max"]
        obj_sense = [objectives_act, sense]
        optimal_values = cbp.utils.optimize.pareto(filename, obj_sense)
        self.assertTrue(len(optimal_values) > 0)

    def test_001_simple_orc(self):
        """Test the simple orc calculation with two storages and 
        heat transfer to the environment."""
        FLUID = "Propane * Butane * Pentane * Hexane"
        comp = [.75, 0.05, 0.15, 0.05]
        comp = [0.4,	0.3,	0.3, 0.0]  # [0.164,.3330,.50300,0.0]

        FLS = "Water"  # Storage fluid
        FLCOLD = "Methanol"  # Storage fluid for low T
        FLENV = "Water"  #

        flm = cbp.fprop.FluidModel(FLUID)
        myFluid = cbp.fprop.Fluid(flm, comp)

        secFlm = cbp.fprop.FluidModel(FLS)
        secFluid = cbp.fprop.Fluid(secFlm, [1.])

        coldFlm = cbp.fprop.FluidModel(FLCOLD)
        coldFluid = cbp.fprop.Fluid(coldFlm, [1.])

        envFlm = cbp.fprop.FluidModel(FLENV)
        envFluid = cbp.fprop.Fluid(secFlm, [1.])

        # Condenser(c) and storage (s), secondary fluids fix all, temperatures(T in K),
        # pressures (p in Pa)
        _ETA_S_ = 0.67  # interesting when changed from 0.69 to 0.65, the efficiency
        # decreases, the reason is the low quality along throtteling then
        _ETA_S_P_ = 0.6  # pump
        _STORAGE_T_OUT_ = cbp._T_SURROUNDING
        _COLD_STORAGE_T_OUT_ = cbp._T_SURROUNDING
        _ENV_T_IN_ = cbp._T_SURROUNDING
        _ENV_T_OUT_ = cbp._T_SURROUNDING + 5.
        _STORAGE_T_IN_ = 363.  # 395.0
        _COLD_STORAGE_T_IN_ = 260.15
        _STORAGE_P_IN_ = 5e5
        _COLD_STORAGE_P_IN_ = 5e5
        _ENV_P_IN_ = 5e5
        _Q_DOT_MIN_ = 1e3  # and heat_flow rate (W)
        _D_T_SUPER_ = 5  # super heating of working fluid
        _D_T_MIN_ = 4.  # minimum approach temperature (pinch point)
        _COP_CHARGING = 3.14  # needed to calculate Q_env_discharging
        _T_REDUCTION_EVAP = -27  # if the curves cross in the evaporator this parameter may help

        # environment for heat transfer
        state_env_out = envFluid.set_state([_ENV_T_OUT_, _ENV_P_IN_], "TP")
        state_env_in = envFluid.set_state([_ENV_T_IN_, _ENV_P_IN_], "TP")

        # high T-storages
        state_sec_out = secFluid.set_state(
            [_STORAGE_T_OUT_, _STORAGE_P_IN_], "TP")
        state_sec_in = secFluid.set_state(
            [_STORAGE_T_IN_, _STORAGE_P_IN_], "TP")

        #  low T sorages:
        state_cold_out = coldFluid.set_state(
            [_COLD_STORAGE_T_OUT_, _COLD_STORAGE_P_IN_], "TP")
        state_cold_in = coldFluid.set_state(
            [_COLD_STORAGE_T_IN_, _COLD_STORAGE_P_IN_], "TP")

        # working fluid

        state_satv_evap = myFluid.set_state(
            [_STORAGE_T_IN_-_D_T_MIN_-_D_T_SUPER_+_T_REDUCTION_EVAP, 1.], "TQ")  # find high pressure
        p_high = state_satv_evap[1]

        T_OUT = _STORAGE_T_IN_ - _D_T_MIN_
        # Evaporator input comes from the pump-output

        state_out_evap = myFluid.set_state([p_high,
                                            T_OUT], "PT")
        # low pressure, condenser # BA 2023-11-14 three points needed: low, environment and slightly higher
        T_SATL = _COLD_STORAGE_T_IN_ + _D_T_MIN_
        state_out_cond = myFluid.set_state(
            [T_SATL, 0.], "TQ")  # find low pressure
        p_low = state_out_cond[1]
        # BA changed 2023-12-13 the fixed starting point for the cycle is the  fluid
        # state before the pump now.

        # the other states in the condenser are fixed by the expander outlet and
        # the Q_total : Q_low_stored ratio    eventually p_low must be varied until
        # the balance is fulfilled, since m_dot_w is fixed by the evaporator!

        FIXED_POINTS = {"eta_s": _ETA_S_,  # expander
                        "eta_s_p": _ETA_S_P_,  # pump
                        "p_low": p_low,
                        "p_high": p_high,
                        "T_hh": _STORAGE_T_IN_,
                        "h_h_out_sec": state_sec_out[2],
                        "h_h_out_w": state_out_evap[2],
                        "h_l_out_cold": state_cold_out[2],
                        "h_l_out_w": state_out_cond[2],
                        "h_env_in": state_env_in[2],
                        "h_env_out": state_env_out[2],
                        "T_hl": _STORAGE_T_OUT_,
                        "T_lh": _COLD_STORAGE_T_OUT_,
                        "T_ll": _COLD_STORAGE_T_IN_,  # 256.0,
                        "Q_dot_h": _Q_DOT_MIN_,
                        "d_temp_min": _D_T_MIN_,
                        "cop_charging": _COP_CHARGING  # needed to calculate Q_env_discharging
                        }

        orc0 = cbp.orc_simple.OrganicRankineCycle(
            [myFluid, secFluid, envFluid, coldFluid], FIXED_POINTS)
        eta_dis = orc0.calc_orc(False)
        self.assertAlmostEqual(0.053007094612255716, eta_dis, places=14)

    def test_002_combine_fluid_screening_data_frames(self):
        """Test the combination of two dataframes function within
        utils.optimize."""

        directory = cbp._CARBATPY_BASE_DIR
        directory += "\\tests\\test_files\\"

        filename1 = directory + r"test_data_ProEthPenBut\2024-02-06-16-51-ProEthPenBut.csv"
        filename2 = directory + \
            r"test_data_ProEthPenBut\2024-02-06-16-51-ProEthPenBut-compressor-Roskosch.csv"
        combined_data = cbp.utils.property_eval_mixture.combine([filename1,
                                                                filename2],
                                                                filename_out="automatic")

        self.assertTrue(len(combined_data) > 0)

    def test_003_plot_data_frame(self):
        """Tplot the columns of a dataframe within
        utils.optimize."""

        directory = cbp._CARBATPY_BASE_DIR
        directory += "\\tests\\test_files\\"

        filename = directory + r"test_data_ProEthPenBut\2024-02-06-16-51-ProEthPenBut.csv"
        what = {"x": 'spec_Volume_sup', "y": 'COP_is80', "hue": 'T_glide_h',
                "size": 'p_ratio', 'style': 'Temperature_hplt'}
        success = cbp.utils.property_eval_mixture.data_plot(filename, what,
                                                            filename_out="automatic")

        self.assertTrue(success)

    def test_004_evaluate_data_(self):
        """Evaluate the data in the dataFrame, when the isentropic efficiencies
        are calculated. Typically using '-combined'-files"""

        directory = cbp._CARBATPY_BASE_DIR
        directory += "\\tests\\test_files\\"
        filename = directory + r"test_data_ProEthPenBut\2024-02-06-16-51-ProEthPenBut-combined.csv"
        data = pd.read_csv(filename)
        initial =len(data.columns)

        data_out = cbp.utils.property_eval_mixture.eval_is_eff_roskosch(data,
                                                                        file_out=directory+"fluid-re-eval.csv")
        self.assertTrue(len(data_out.columns) > initial)
        
    def test_005_curve_dist_find(self):
        """Evaluate the data in the dataFrame, when the isentropic efficiencies
        are calculated. Typically using '-combined'-files"""

        x_val = np.linspace(1, 12)
        # test points
        y_val = -5*np.sin(x_val) + 5 * x_val + 4 - 100/x_val
        values_act = np.array([x_val, y_val])
        solution_below = cbp.curve_min_distance_finder.find_min_approach(values_act, True)
        solution_above = cbp.curve_min_distance_finder.find_min_approach(values_act, False)
        self.assertTrue(solution_below["success"])
        self.assertTrue(solution_above["success"])
        self.assertAlmostEqual(solution_below["integral"],
                               356.82447354719505, places=14)


if __name__ == "__main__":
    unittest.main()
